<?php
/******************************
/ class.recurrences.php
/ functions for handling event recurrence
/
/ We are going to have a number of default recurrence settings. They are...
/
/
Definition								Offset			Notes								
-----------------------------------------------------------------------------------------------
No repeat								n/a				Default.								
Every [day of the week]					7 days			Get current day of week; if same, set for next 7 days.										
On the [x] day of every month			1 month			Can't allow day to be > 28				
On the [1-4] [day of week] of month		1 month			Must calculate that date every time		
Every [date] of each year				1 year			Add 1 year out.

We need to calculate the NEXT date for recurrence, based on the date handed to the function.

All functions will return the NEXT recurrence date for the given date.
*/


class Recurrences {
	
	var $input_date;
	var $day_of_week;
	var $num_weeks;
	var $num_day_of_week;
	var $day_of_month;
	var $which_weekday;
	
	function Recur_Weekly() {
		// take $day_of_week, compare with $input_date.
		// if $input_date IS the day given, simply add 7 days and return.
		// else, find the next $day_of_week, and return it
		$oneweek = 604800; //seconds in a week
		
		$day = tz_date("l", $this->input_date); // gives the day of the week of the input date
		// is the event going to happen today or has it passed?
		if($day == $this->day_of_week) {
			//echo $this->input_date;
			if($this->input_date > time()) {
				// we have the right day of the week and the event is happening later today
				// return the time we're given.
				return $this->input_date;
			}
			else {
				// the event happens on this day of the week but it's passed today. 
				// return this day next week
				return $this->input_date + $oneweek;
			}
			
		}
		else {
			// cycle through the next 6 days; one of them has to match!
			$day_to_match = $this->input_date;
			for($i=0; $i<7; $i++) {
				if($this->day_of_week == tz_date("l", $day_to_match + 86400)) { //seconds in a day
					// this day matches the desired day; return the date
					$day_to_match = $day_to_match + 86400;
					return $day_to_match;
					exit;
				}
				else {
					$day_to_match = $day_to_match + 86400;
				}
			}
			
		}
	}
	
	function Recur_Day_Monthly() {
		// return an error for $day_of_month > 28 because this would break in February!		
		if($this->day_of_month > "28") {

			return false;
		}
		else {
					
			$day = tz_date("j", $this->input_date);
			
			if($this->day_of_month == tz_date("j", $this->input_date)) {
				// has this event happened today? Or will it happen later today?
				if($this->input_date > time()) {
					// this event takes place on this day of the month, and later than now.
					// Return the date we've been given
					return $this->input_date;
				}
				else {
					// we have the right day of the month, but the event has already passed for today
					// return the date plus one month
					return strtotime("+1 month", $this->input_date);
				}
			}
			else {
				//cycle through the next 30 days; one of them has to match!
				$day_to_match = $this->input_date;
				
				for($i=0; $i<31; $i++) {
					if($this->day_of_month == tz_date("j", $day_to_match + 86400)) {
					// this day matches the desired day; return the date
					$day_to_match = $day_to_match + 86400;
					return $day_to_match;
					exit;
				}
				else {
					$day_to_match = $day_to_match + 86400;
				}
				}
			}
			
		}
		
	}
	
	function Recur_Weekday_Monthly() {
		// Call Mr Motley's nthDayofMonth function... sweet.

		$day = $this->nthDayOfMonth($this->which_weekday, $this->num_day_of_week, tz_date("m", $this->input_date), tz_date("Y", $this->input_date));
		if($day < 10) {
			$day = "0".$day;
		}
		//preserve the time of day info from the input_date so we can append it to the datestring
		$time = tz_date("H:i:s", $this->input_date);
		
		$datestring = tz_date("Y", $this->input_date).date("m", $this->input_date).$day." ".$time;
		$datestring = strtotime($datestring);
	
		//if this date has already passed, then let's find the one for next month
		if($datestring < $this->input_date) {
			$day = $this->nthDayOfMonth($this->which_weekday, $this->num_day_of_week, tz_date("m", $this->input_date)+1, tz_date("Y", $this->input_date));
			if($day < 10) {
				$day = "0".$day;
			}
			$nextmonth = tz_date("m", $this->input_date) + 1;
			
			if($nextmonth < 10) {
				$nextmonth = "0".$nextmonth;
			}
			
			$datestring = tz_date("Y", $this->input_date).$nextmonth.$day." ".$time;
			$datestring = strtotime($datestring);
		}
		
		return $datestring;
		
	}
	function call_nth_day() {
		$day = $this->nthDayOfMonth($this->which_weekday, $this->num_day_of_week, tz_date("m", time()), tz_date("Y", time()));
		return $day;
	}
	
	function Recur_n_Weekly() {
			// take $input_date and $num_weeks.
			// if now() is greater than $input_date, return $input_date + $num_weeks
			// else return $input_date
			// added in version 0.3 beta
			
			$weeks_to_add = $this->num_weeks * 604800; //seconds in a week

			if(time() > $this->input_date) {
				return $this->input_date + $weeks_to_add;
			}
			else {
				return $this->input_date;
			}
		}
	

	function Recur_Annually() {
		
		// if the event happens today, has it happened yet?
		if($this->input_date > time()) {
			return $this->input_date;
		}
		else {
			return strtotime("+1 year", $this->input_date); 			
		}		
	}

/******************************
/ PRIVATE FUNCTIONS TO THIS CLASS
/ Thanks for Mark Motley -- I found it, after much useless coding of my own and hours of frustration, here:
/ http://www.phpbuilder.com/snippet/detail.php?type=snippet&id=293
/ THIS CODE IS LICENSED UNDER THE GPL
	
	/*
	 * PHP Day of Week functions
	 * Copyright (c) 2001, Mark Motley <mmotley@la-mirada.net>
	 * GNU Public license, use as you want
	 */

	/*
	 * dayofweek($day,$month,$year) will compute the day of the week a
	 * a particular date falls.
	 * 
	 * Paramaters:
	 *
	 *   $day - Day of month [1-31]
	 *   $month - Month [1-12]
	 *   $year - The year [1900-?]
	 *
	 * Returns:
	 *
	 *   Numerical day of week where 0=Sunday, 1=Monday, etc.
	 *   On error (out-of-bounds dates) returns -1.
	 *
	 */

	function dayofweek($day,$month,$year) {

	        /* Check date for validity */
	        if (!checkdate($month,$day,$year))
	                return -1;

	        $a=(int)((14-$month) / 12);
	        $y=$year-$a;
	        $m=$month + (12*$a) - 2;

	        $retval=($day + $y + (int)($y/4) - (int)($y/100) + (int)($y/400) + (int)((31*$m)/12)) % 7;

	        return $retval;
	}

	/* phpdow_mod($a,$b) is a fixed-up modulo function that will deal with
	 * negative numbers properly. PHP's modulo function doesn't, and the
	 * forumulas are dependent on it.
	 * It's really only here to support the nthDayOfMonth() function.
	 */

	function phpdow_mod($a,$b) {
	        if ($a <= 0)
	                return (int)$this->phpdow_mod($b-abs($a),$b);
	        else
	                return (int)($a%$b);
	}


	/*
	 * nthDayOfMonth($n,$dow,$month,$year) will compute the Nth day of the given
	 * month.  For example, the first Monday in April 2001.
	 *
	 * Parameters:
	 *
	 *  $n - the Nth day you want, i.e. 2 for 2nd 
	 *  $dow - The day of week you want, 0=Sunday, 1=Monday, etc. [0-6]
	 *  $month - The month you want [1-12]
	 *  $year - The full year [1900? - ??]
	 *
	 * Returns:
	 * 
	 *  The date, in the month you passed, that fits the criteria.
	 *  Here we return to error conditions:
	 *    -1 = invalid date
	 *    -2 = There is no Nth day of that month, like no 5th Tuesday
	 *         of the specified month
	 *
	 * Example:
	 *
	 *  To find the first Tuesday in June 2001:
	 *     $day = nthDayInMonth(1,2,6,2001);
	 *  will return 4, which means the first Tuesday in June is 6/4/2001.
	 *
	 */

	function nthDayOfMonth($n,$dow,$month,$year) {

	        /* Check the date */
	        if ($month > 12) 
	                return -1;

	        if ($dow > 6)
	                return -1;

	        /* Valid Nth day, should be no more than 5 */
	        if (($n <= 0) || ($n > 5))
	                return -1;

	        $retval = (7*$n)-6+$this->phpdow_mod($dow-$this->dayofweek(1,$month,$year),7);

	        /* Make sure the date is not greater than the number of days
	           in the month */
	        if (!checkdate($month,$retval,$year))
	                return -2;
	
	        /* Otherwise we return the date */
	        return $retval;
	}
	
}


?>